package com.metawear.mobility;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.le.BluetoothLeScanner;
import android.bluetooth.le.ScanCallback;
import android.bluetooth.le.ScanResult;
import android.os.Handler;
import android.os.ParcelUuid;
import android.view.View;

import java.util.HashSet;
import java.util.UUID;


class BlueToothScanner {

    private MainActivity mainActivity;
    private BluetoothAdapter bluetoothAdapter;
    private BluetoothLeScanner bluetoothLeScanner;
    private boolean isScanning;
    private Handler handler;
    private HashSet<ParcelUuid> m_filterServiceUuids;

    // Stops scanning after 10 seconds.
    private static final long SCAN_PERIOD = 7000;

    BlueToothScanner(MainActivity mainActivity, BluetoothAdapter bluetoothAdapter) {
        this.bluetoothAdapter = bluetoothAdapter;
        this.bluetoothAdapter.enable();
        bluetoothLeScanner = this.bluetoothAdapter.getBluetoothLeScanner();
        this.mainActivity = mainActivity;

        UUID[] filterUuids = new UUID[]{UUID.fromString("326a9000-85cb-9195-d9dd-464cfbbae75a")};//commBus.getFilterServiceUuids();
        m_filterServiceUuids = new HashSet<>();
        for (UUID uuid : filterUuids) {
            m_filterServiceUuids.add(new ParcelUuid(uuid));
        }

    }

    void scanLeDevices() {
        handler = new Handler();
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                DeviceList.getInstance().clearList();

                // Stops scanning after a pre-defined scan period.
                handler.postDelayed(() -> {
                    isScanning = false;
                    bluetoothLeScanner.stopScan(leScanCallback);
                    System.out.println("scan finished!");

                    int index = 0;
                    while (true) {
                        if (MetawearServices.lastTaskIsFinished) {
                            MetawearServices.lastTaskIsFinished = false;
                            MetawearServices.getInstance().retrieveBoard(index, Operation.SCAN_DEVICE, null);
                            index ++;
                        }

                        if (index == DeviceList.getInstance().getMetawearList().size())
                            break;

                        try {
                            Thread.sleep(500);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                        }
                    }
                }, SCAN_PERIOD);

                isScanning = true;
                bluetoothLeScanner.startScan(leScanCallback);
            }
        });

        thread.start();
    }

    // Device scan callback.
    private ScanCallback leScanCallback = new ScanCallback() {
        @Override
        public void onScanResult(int callbackType, final ScanResult result) {
            super.onScanResult(callbackType, result);

            if (result.getScanRecord() != null && result.getScanRecord().getServiceUuids() != null) {
                boolean valid = true;
                for (ParcelUuid it : result.getScanRecord().getServiceUuids()) {
                    valid &= m_filterServiceUuids.contains(it);
                }

                if (valid && !deviceExists(result.getDevice().getAddress()))
                    mainActivity.runOnUiThread(() -> {
                        DeviceList.getInstance().addDevice(result.getDevice().getAddress());
                        DeviceList.getInstance().notifyDataSetChanged();
                    });
            }
        }
    };

    private boolean deviceExists(String address) {
        for (String mac : DeviceList.getInstance().getMetawearList()) {
            if (mac.equals(address)) {
                return true;
            }
        }

        return false;
    }
}



